/////////////////////////////////////////////////////////////////////////
////                          PID.C                                  ////
////                                                                 ////
////  Programme de gestion de deux PID (asservissemnt de vitesse)    ////
////                 16F877 quartz 20Mhz                             ////
////  Auteur: 	Fribottes					     ////
////  Date:	 15 fevrier 2003                                     ////
////                                                                 ////
/////////////////////////////////////////////////////////////////////////
////        (C) Copyright 2002 Fribotte 		             ////
////								     ////
/////////////////////////////////////////////////////////////////////////

#include <16F877.H>

#fuses  HS,NOWDT,NOPROTECT, PUT, NOLVP

#use delay(clock=20000000)							// A configurer en fonction de votre quartz
										// il se peut que des tempos soit  revoir.

#use rs232(baud=9600, xmit=PIN_C6, rcv=PIN_C7, parity=N, bits=8)

										
#define all_out 0x00; /* port en sortie*/
#define all_in 0x01; /* port en entree*/

#define SET	PIN_B7								// Definition de la touche SET
#define CANCEL	PIN_B6								// definition de la touche CANCEL
#define PLUS	PIN_B5								// definition de la touche PLUS
#define MOINS	PIN_B4								// definition de la touche MOINS

/* definition des entres sortie*/
#byte portA=5							// adresse du port A
#byte portB=6							// adresse du port B
#byte portC=7							// adresse du port C
#byte portD=8							// adresse du port D
#byte portE=9							// adresse du port E


#byte LCD_DATA_PORT=portD

// definition de chaine globale de caractere
char string[21];
// 20x4 module
   //  Line 1 addresses are 00h to 13h
   //  Line 2 addresses are 40h to 53h
   //  Line 3 addresses are 14h to 27h
   //  Line 4 addresses are 54h to 67h


// Definition emplacement EEPROM
#define EEPROM_KP_MOTEUR1 		0
#define EEPROM_KP_MOTEUR2 		1

#define EEPROM_INVKI_MOTEUR1 		2
#define EEPROM_INVKI_MOTEUR2 		3

#define LIMITE_SI_MOTEUR1_fort		4
#define LIMITE_SI_MOTEUR1_faible	5

#define LIMITE_SI_MOTEUR2_fort		6
#define LIMITE_SI_MOTEUR2_faible	7

#define LIMITE_MESURE_HAUT_MOTEUR1	8
#define LIMITE_MESURE_BAS_MOTEUR1	9

#define LIMITE_MESURE_HAUT_MOTEUR2	10
#define LIMITE_MESURE_BAS_MOTEUR2	11


// definition des fonctions:
void InitLcd();							// initialisation de l ecran
void Lcd_Line1();						// position en debut de ligne 1
void Lcd_Line2();						// position en debut de ligne 2
void Lcd_Line3();						// position en debut de ligne 3
void Lcd_Line4();						// position en debut de ligne 4
void Lcd_Send_noWait(char VarSend);				// envoi de commande dans delais
void Lcd_Send(char VarSend);					// envoi de commande avec delais de 50us
void Lcd_Hexa(char VarSend);					// affichage d une valeur hexa 0x..
void Lcd_Binaire(char VarSend);					// Affichage d une valeur Binaire 10101010b
void Lcd_Decimal(char ValSend);					// Affichage d une valeur en decimal (1 octet)
void Lcd_Clear();						// Effacement de l ecran et position du curseur en 00
void Lcd_Char(char CharSend);					// Affichage du un caractere, passe la valeur Ascii du carac
void ReturnHome();						// revient en position 00
void Lcd_Printf(char *Chaine);					// Affichage d une chaine de caractere
void Lcd_Cur_S_Right();						// decalage du curseur vers la droite
void Lcd_Cur_S_Left();						// decalage du curseur vers la gauche
void AffichageAlphabet();					// Affichage de toutes les caracteres supports par le LCD
void DisplayHexaBinDeci(char Val);				// Affichage d une valeur en Hexa, binaire, decimal pour ecran 4 ligne
void Menu();							// definition du menu
// gestion du clavier
char GetKey();							// recuperation d un caractere.
char GetString();						// recuperation d une chaine de caractere.
// fonction delais
void delay_sec(int n);						// programmation d un delay de 1 seconde

// Port definition
#byte	PORTA = 5
#byte	PORTB = 6
#byte	PORTC = 7	
#byte	PORTD = 8
#byte	PORTE = 9


// PWM
void SET_PWM1_DUTY_10b (signed long PWM1);
void SET_PWM2_DUTY_10b (signed long PWM2);

#SEPARATE void compteur();		// Gestion du compteur

short int interruption;
int compteurBoucle;
short int temps;

int  lastValueCompteurInter_1;
int  lastValueCompteurInter_2;
short int mesure_sens_1;
short int mesure_sens_2;

#SEPARATE void calculPID();

// Compteurs
signed long  compteur_1;
signed long  compteur_2;

// Compteurs globaux (pour retour par liaison serie)
signed long  compteur_1_global;
signed long  compteur_2_global;


// Consigne
int	consigne1; // En pas par base de temps
int	consigne2;
short int 	sensConsigne1; // Sens 
short int 	sensConsigne2;
short int freinMoteur1;
short int freinMoteur2;
short int roueLibreMoteur1 = 1;
short int roueLibreMoteur2 = 1;
int BaseTempsMoteur1 = 20; // 10ms defaut
int BaseTempsMoteur2 = 20;
int BaseTempsMoteurCmp1 = 1;
int BaseTempsMoteurCmp2 = 1;

// Reception des commandes
int statusRecu = 0;
int recu1;
int recu2;
int recu3;

// Pour que les emissions soient pas plus rapide que toutes les ms.
int emitBuffer;

// buffer d'emission
int bufferEmission0;
int bufferEmission1;
int bufferEmission2;
int bufferEmission3;
int sizeBuffer;

// Pour le retour de l'evolution du PID
int EnvoisErreur;
short int EnvoisErreurPid;


// Infos compteur 1
/*
int infocompteur1_1;
int infocompteur1_2;
int infocompteur1_3;
int infocompteur1_4;
*/

// retour de certaines infos 
signed long lastPWM1;
signed long lastPWM2;
signed long lastError1;
signed long lastError2;

// Somme des erreurs
signed long Si1;
signed long Si2;

short int pid;

/*
void convertisseurAN()
{
	long i;
	int j;

	i = READ_ADC();
	j = (char)(i & 0xFF);
	j >>= 2;

	if ( ((char)(i>>8)) &1) 
		j |= 64;
	if ( ((char)(i>>8)) &2) 
		j |= 128;

	if (j>127)
		j = 0;
	else
		j = tableauDistace[j];

	compteur_convertisseur++;
	if (compteur_convertisseur==1) { 
		SET_ADC_CHANNEL(1);
		retourDistanceMilieu = j;
	}
	if (compteur_convertisseur==2) { 
		SET_ADC_CHANNEL(3);
		retourDistanceGauche = j;
	}
	if (compteur_convertisseur==3) { 
		SET_ADC_CHANNEL(0);
		retourDistanceDroite = j;
		compteur_convertisseur = 0;
	}

}
*/


// ************************
// ****  Interruptions ****
// ************************

// Toutes les 0.5 ms
#INT_TIMER2
void timer2_interrupt() {

	int valueCompteur;
	int newValueCompteur;
	int limiteHaute;

	interruption = 1;

	// buffer d'emission
	if (emitBuffer!=0)
		emitBuffer--;

	// Toutes les 0.5ms, calcul de la vitesse des moteurs
	//  Et arrete leur mesure si ils vont trop vite

	limiteHaute = READ_EEPROM(LIMITE_MESURE_HAUT_MOTEUR1);
	if (limiteHaute) {

		// Compteur 1 
		newValueCompteur = GET_RTCC();
		if (newValueCompteur>=lastValueCompteurInter_1) {
			valueCompteur = newValueCompteur - lastValueCompteurInter_1;
		} else {
			valueCompteur = 256 - lastValueCompteurInter_1 + newValueCompteur;
		}

		if (mesure_sens_1) {
			if (valueCompteur>limiteHaute)
				mesure_sens_1 = 0;
		}
		else {
			if (valueCompteur<READ_EEPROM(LIMITE_MESURE_BAS_MOTEUR1))
				mesure_sens_1 = 1;
		}
		lastValueCompteurInter_1 = newValueCompteur;
	}

	// Compteur 2
	limiteHaute = READ_EEPROM(LIMITE_MESURE_HAUT_MOTEUR2);
	if (limiteHaute) {
		newValueCompteur = GET_TIMER1();
		if (newValueCompteur>=lastValueCompteurInter_2) {
			valueCompteur = newValueCompteur - lastValueCompteurInter_2;
		} else {
			valueCompteur = 256 - lastValueCompteurInter_2 + newValueCompteur;
		}
		if (mesure_sens_2) {
			if (valueCompteur>limiteHaute)
				mesure_sens_2 = 0;
		}
		else {
			if (valueCompteur<READ_EEPROM(LIMITE_MESURE_BAS_MOTEUR2))
				mesure_sens_2 = 1;
		}
		lastValueCompteurInter_2 = newValueCompteur;	
	}


	// calcule l'un des PID si necessaire

	if (BaseTempsMoteurCmp1 == BaseTempsMoteurCmp2) {
		BaseTempsMoteurCmp2++;  // on les decale de 0.5 ms pour que les 2 calculs de PID ne se fassent pas 
						//  en meme temps. N'arrive qu'une fois, en debut de cycle, 
						//  les BaseTempsMoteurX etant forcement pair.
	}

	BaseTempsMoteurCmp1--;
	if (BaseTempsMoteurCmp1 == 0) {
		if (temps ==0) {
			pid = 1;
			temps = 1;
			compteurBoucle = 128;
			BaseTempsMoteurCmp1 = BaseTempsMoteur1;		
		}
		else 
			BaseTempsMoteurCmp1++;
	}

	BaseTempsMoteurCmp2--;
	if (BaseTempsMoteurCmp2 == 0) {
		if (temps ==0) {
			pid = 0;
			temps = 1;
			compteurBoucle = 128;
			BaseTempsMoteurCmp2 = BaseTempsMoteur2;
		}
		else 
			BaseTempsMoteurCmp2++;
	}


}


// ************************
// ** Fonction principal **
// ************************
main() {
	char answer;
	char memval;
	int i;

	#BIT T1_ACTIVE = 0x10.3	
	#BYTE T1 = 0x10

	// init buffer
	sizeBuffer = 0;
	emitBuffer = 0;

	// retour PID
	EnvoisErreur = 0;

	//compteur_convertisseur = 0;
	lastValueCompteurInter_1 = 0;
	lastValueCompteurInter_2 = 0;
	mesure_sens_1 = 1;
	mesure_sens_2 = 1;
	//debug = 1;
	
	delay_ms(15); 							/* boucle de 15 ms attente necessaire apres On*/
	SET_TRIS_D(0x00); 						/* Mise en sortie du port D pour le LCD*/
	SET_TRIS_B(0xFF); 						// Port B en entre

	for (i=0; i<10; i++) {
		OUTPUT_HIGH(PIN_D3);
		delay_ms(100);
		OUTPUT_LOW(PIN_D3);
		delay_ms(100);
	}

	//InitLcd();							// Initialisation de l ecran.

	SET_TRIS_C(1+128);

	/*SETUP_TIMER_1(T1_EXTERNAL);*/	
	// Configure le timer1
	//T1_ACTIVE = 1;
	T1 = 3;

	#USE FAST_IO(A)
	#USE FAST_IO(B)
	#USE FAST_IO(C)

	SETUP_COUNTERS(RTCC_EXT_L_TO_H, WDT_18MS);
	SET_TRIS_A(0xFF);
	SET_TRIS_B(0);

	// Convertisseur analogique/numerique
	SETUP_ADC_PORTS(RA0_RA1_RA3_ANALOG);
	SETUP_ADC( ADC_CLOCK_INTERNAL );
	// juste le 1er
	SET_ADC_CHANNEL(0);

	setup_ccp1(CCP_PWM);
	setup_ccp2(CCP_PWM);
	setup_timer_2(T2_DIV_BY_1, 249, 10);
	SET_PWM1_DUTY_10b(0);
	SET_PWM2_DUTY_10b(0);

	ENABLE_INTERRUPTS(INT_TIMER2);
	ENABLE_INTERRUPTS(GLOBAL);

	consigne1 = 0;
	consigne2 = 0;
	sensConsigne1 = 1;
	sensConsigne2 = 0;
	compteur_1_global = 0;
	compteur_2_global = 0;

	lastPWM1 = 0;
	lastPWM2 = 0;
	lastError1 = 0;
	lastError2 = 0;

	compteur();

}

#INLINE void RS232_INPUT() 
{
	int recu;

	static int cmpRecu =0;
	static int cmpLED =0;

	short int renvoisRecu = 0;
	short int erreurRecu = 0;

	#BIT OERR = 0x18.1
	#BIT CREN = 0x18.4

	// Pour l'ecriture EEPROM
	#BYTE EECON1 = 0x18C
	#BIT  EECON1_EEPGD = 0x18C.7
	#BIT  EECON1_WREN = 0x18C.2
	#BIT  EECON1_WR = 0x18C.1
	#BIT  EECON1_RD = 0x18C.0
	#BYTE EECON2 = 0x18D
	#BYTE EEDATA = 0x10C
	#BYTE EEADR = 0x10D


	if (cmpLED!=0) {
		cmpLED--;
		OUTPUT_HIGH(PIN_D3);
	} 
	else
		OUTPUT_LOW(PIN_D3);

	if (sizeBuffer !=0 ) {
		if (emitBuffer==0) { // to wait at least 1ms between each send
			switch(sizeBuffer) {
				case 4: // Envois bufferEmission0
					PUTC(bufferEmission0);
				break;
				case 3: // Envois bufferEmission1
					PUTC(bufferEmission1);
				break;
				case 2: // Envois bufferEmission2
					PUTC(bufferEmission2);
				break;
				case 1: // Envois bufferEmission1
					PUTC(bufferEmission3);
				break;
			}
			sizeBuffer--;
			emitBuffer = 2; // Attends 2*0.5ms pour la prochaine emission
		}
		statusRecu = 0;
		return;
	}

	while (KBHIT()) {

		cmpRecu++;
		cmpLED = 255;

		recu = getc();

		switch(statusRecu) {
			case 0: // attends un 0
				if (recu ==0)
					statusRecu = 1;
			break;

			case 1: // la commande

				recu1 = recu;
				statusRecu = 2; // Prochain sur le 2 (pas de test de validation de la commande)

				if (recu1 ==255) {
					// Frein d'urgence sur les 2 moteurs
					freinMoteur1 = 1;
					freinMoteur2 = 1;
					roueLibreMoteur1 = 0;
					roueLibreMoteur2 = 0;
					break;
				}
				if (recu1 ==254) {
					// Roue libre sur les 2 moteurs			
					freinMoteur1 = 0;
					freinMoteur2 = 0;
					roueLibreMoteur1 = 1;
					roueLibreMoteur2 = 1;
					break;
				}
			break;

			case 2: // 1er octet de donne
				recu2 = recu;
				statusRecu = 3; // attends le 2eme octet de donnee
			break;

			case 3: // 2er octet de donne
				recu3 = recu;
				statusRecu = 0; // Apres ce traitement on revient au debut
			
				// renvois un accuse de reception
				renvoisRecu = 1;					

				// traitement moteur 1
				if (recu1 == 1 || recu1==3 || recu1==4 || recu1==51) {
					freinMoteur1 = 0;
					roueLibreMoteur1 = 0;
					if (recu2==255) {
						freinMoteur1 = 1;
					} else {
						if (recu2==254) {
							roueLibreMoteur1 = 1;
						} else {
							BaseTempsMoteur1 = recu2 & 254;
							sensConsigne1 = 0;
							if (recu2 & 1) 
								sensConsigne1 = 1;
							consigne1 = recu3;
						}
					}
				}

				// traitement moteur 2
				if (recu1 == 2 || recu1==3 || recu1==4 || recu1==52) {
					freinMoteur2 = 0;
					roueLibreMoteur2 = 0;
					if (recu2==255) {
						freinMoteur2 = 1;
					} else {
						if (recu2==254) {
							roueLibreMoteur2 = 1;
						} else {
							BaseTempsMoteur2 = recu2 & 254;
							if (recu1!=4) {
								sensConsigne2 = 0;
								if (recu2 & 1) 
									sensConsigne2 = 1;
							} else {
								sensConsigne2 = 1;
								if (recu2 & 1) 
									sensConsigne2 = 0;
							}
							consigne2 = recu3;
						}
					}
				}
				if (recu1 == 51) {
					EnvoisErreur = 255;
					EnvoisErreurPid = 1;
				}
				if (recu1 == 52) {
					EnvoisErreur = 255;
					EnvoisErreurPid = 0;
				}

				// traitement reprogramation de l'EEPROM
				if (recu1 == 10) {

					// Test d'abord si la derniere ecriture EEPROM a bien finie
					if (EECON1_WR) {
						renvoisRecu = 0;
						erreurRecu = 1;
					}
					else {
						EEDATA = recu3;
						EEADR = recu2;
						EECON1_EEPGD = 0;
						EECON1_WREN = 1;
					
						DISABLE_INTERRUPTS(GLOBAL);
						EECON2 = 0x55;
						EECON2 = 0xAA;
						EECON1_WR = 1;
						ENABLE_INTERRUPTS(GLOBAL);
					}
				}
				if (recu1 == 11) {
					renvoisRecu = 1;
					sizeBuffer = 1;
					// buffer d'emission
					bufferEmission3 = READ_EEPROM(recu2);
				}
				if (recu1 == 20) {
					renvoisRecu = 1;
					sizeBuffer = 4;
					// buffer d'emission
					bufferEmission0 = compteur_1_global>>8;
					bufferEmission1 = compteur_1_global & 0xFF;
					bufferEmission2 = compteur_2_global>>8;
					bufferEmission3 = compteur_2_global & 0xFF;
				}
				if (recu1 == 30) {
					renvoisRecu = 1;
					sizeBuffer = 4;
					// buffer d'emission
					bufferEmission0 = lastPWM1 >>8;
					bufferEmission1 = lastPWM1 & 0xFF;
					bufferEmission2 = lastPWM2 >>8;
					bufferEmission3 = lastPWM2 & 0xFF;
				}
				if (recu1 == 31) {
					renvoisRecu = 1;
					sizeBuffer = 4;
					// buffer d'emission
					bufferEmission0 = lastError1 >>8;
					bufferEmission1 = lastError1 & 0xFF;
					bufferEmission2 = lastError2 >>8;
					bufferEmission3 = lastError2 & 0xFF;
				}
/* Desactive
				if (recu1 == 40) {
					renvoisRecu = 1;
					sizeBuffer = 4;
					// buffer d'emission
					bufferEmission0 = infocompteur1_1;
					bufferEmission1 = infocompteur1_2;
					bufferEmission2 = infocompteur1_3;
					bufferEmission3 = infocompteur1_4;
				}
*/				
				if (recu1 == 60) {
					renvoisRecu = 1;
					sizeBuffer = 1;
					// buffer d'emission
					if (recu2==1)
						bufferEmission3 = PORTA;
					if (recu2==2)
						bufferEmission3 = PORTB;
					if (recu2==3)
						bufferEmission3 = PORTC;
					if (recu2==4)
						bufferEmission3 = PORTD;
					if (recu2==5)
						bufferEmission3 = PORTE;
				}
			break;
		}
	}
	if (EECON1_WREN && (!EECON1_WR))
		EECON1_WREN = 0;

	// Oveflow sur le buffer d'entree ...
	if (OERR) {
		// reset reception
		CREN = 0;
						
		// init
		statusRecu = 0;
		renvoisRecu = 0;
		erreurRecu = 0;

		// reset reception
		CREN = 1;
	
		if (OERR) {
			while (1) {
				OUTPUT_HIGH(PIN_D3);
				delay_sec(5);
				OUTPUT_LOW(PIN_D3);
				delay_sec(5);	
				};	
		}

		// send error message
		PUTC(255);
		emitBuffer = 2; // Attends 2*0.5ms pour la prochaine emission
	}

	if (renvoisRecu) {
		// send ok message
		PUTC(0);
		emitBuffer = 2; // Attends 2*0.5ms pour la prochaine emission
	}
	if (erreurRecu) {
		// send ko message
		statusRecu = 0; // Apres ce traitement on revient au debut
		erreurRecu = 0;
		PUTC(255);
		emitBuffer = 2; // Attends 2*0.5ms pour la prochaine emission
	}

}


#SEPARATE void compteur()
{

	/* temp */
	unsigned long j;

	// Compteur ...
	static short int sens_1;
	static short int sens_2;
	static short int changementSens_1;
	static short int changementSens_2;
	static int  lastValueCompteur_1;
	static int  lastValueCompteur_2;
	int valueCompteur;
	int  newValueCompteur;
	int signal;
	static int old_signal;
	static int old_old_signal; // temp
	short int sensTmp;
	int xorSignal;
	int xorSignal2;

	sens_1 = 0;
	sens_2 = 0;
	changementSens_1 = 0;
	changementSens_2 = 0;
	compteur_1 = 0;
	compteur_2 = 0;
	lastValueCompteur_1 =0;
	lastValueCompteur_2 =0;
	SET_RTCC(0);
	SET_TIMER1(0);
	old_signal = 0;
	old_old_signal = 0;
	temps = 0;

	interruption = 1;
	
	do {

		do {

			compteurBoucle = 0;
			do {

				DISABLE_INTERRUPTS(GLOBAL);
			        signal = PORTA; // Signal 1,1 A4  Signal 1,2 A5 -  Signal 2.1 C0 Signal 2.2 A2
			        if (INPUT(PIN_C0)) BIT_SET(signal, 0);
				ENABLE_INTERRUPTS(GLOBAL);

			        /* xorSignal : 
			                Bit 6 : Sens 1 actuel
			                Bit 5 : Difference ancien/nouveau sur le signal 2 du compteur 1
			                Bit 4 : Difference ancien/nouveau sur le signal 1 du compteur 1
			                Bit 2 : Difference ancien/nouveau sur le signal 2 du compteur 2
			                Bit 1 : Sens 2 actuel
			                Bit 0 : Difference ancien/nouveau sur le signal 1 du compteur 2
			        */

			        signal &= 0b00110101;
			        old_signal &= 0b00110101;

			        if (BIT_TEST(signal ,4))
			                BIT_SET(signal ,6); 
			        if (BIT_TEST(signal ,5))
			                BIT_SET(Old_signal ,6); 

			        if (BIT_TEST(signal ,0))
			                BIT_SET(signal ,1);
			        if (BIT_TEST(signal ,2))
			                BIT_SET(Old_signal ,1);

			        xorSignal = signal^old_signal;

			        if (BIT_TEST(xorSignal  ,6))
			                BIT_SET(signal ,7);
			        if (sens_1)
			                BIT_SET(Old_signal ,7); // On en a besoin car le sens n'est pas forcement mesure a chaque fois

			        if (BIT_TEST(xorSignal  ,1))
			                BIT_SET(signal ,3);
			        if (sens_2)
			                BIT_SET(Old_signal ,3); // On en a besoin car le sens n'est pas forcement mesure a chaque fois

			        /* xorSignal2 : 
			                Bit 7 : Difference ancien/nouveau sur le sens 1
			                Bit 3 : Difference ancien/nouveau sur le sens 2
			        */

			        xorSignal2 = signal^old_signal;

				if (!interruption)  {      
					if (mesure_sens_1) {
				                if ( BIT_TEST(xorSignal ,4) ) {
				                        if (!BIT_TEST(xorSignal,5)) {
				                                sens_1 = BIT_TEST(xorSignal,6); // Xor
				                                if (BIT_TEST(xorSignal2, 7)) {
			        	                                changementSens_1 = 1;
			                	                        compteurBoucle = 128;
			                        	        }                       
				                        }
				                } 
					}
					if (mesure_sens_2) {
				                if (BIT_TEST(xorSignal,0)) {
				                        if (!BIT_TEST(xorSignal,2)) {
				                                sens_2 = BIT_TEST(xorSignal,1); // Xor
				                                if (BIT_TEST(xorSignal2, 3)) {
			        	                                changementSens_2 = 1;
			                	                        compteurBoucle = 128;
			                        	        }                       
				                        }
				                }
					}
				}
				old_old_signal = old_signal;
			        old_signal = signal;
			        compteurBoucle++;
				interruption = 0;

			} while (!BIT_TEST(compteurBoucle,7));

			// Compteur 1 
			newValueCompteur = GET_RTCC();
			if (newValueCompteur>=lastValueCompteur_1) {
				valueCompteur = newValueCompteur - lastValueCompteur_1;
			} else {
				valueCompteur = 256 - lastValueCompteur_1 + newValueCompteur;
			}

	
			// Pour du debug sur le moteur 1, desactive
/*
			infocompteur1_1 = signal ;//newValueCompteur;
			infocompteur1_2 = old_old_signal ; //lastValueCompteur_1;
			infocompteur1_3 = valueCompteur;
			if (sens_1)
				infocompteur1_4 = 128;
			else
				infocompteur1_4 = 0;
			if (changementSens_1)
				infocompteur1_4 += 64;
			if (mesure_sens_1)
				infocompteur1_4 += 1;
*/

			sensTmp = 0;
			if (changementSens_1) {
				sensTmp = 1;		
				if (BIT_TEST(signal ,4)) { /* Car le compteur compte sur un front montant */
					valueCompteur--; //valueCompteur>0 car on vient incrementer
					if (valueCompteur == 255) {
						sensTmp = 0;
						valueCompteur = 1;						
					}
				}
			}
				
			if (sens_1 != sensTmp) 
				compteur_1 += valueCompteur;				
			else
				compteur_1 -= valueCompteur;

			lastValueCompteur_1 = newValueCompteur;
			

			changementSens_1 = 0;

			// Compteur 2 
			newValueCompteur = GET_TIMER1();
			if (newValueCompteur>=lastValueCompteur_2) {
				valueCompteur = newValueCompteur - lastValueCompteur_2;
			} else {
				valueCompteur = 256 - lastValueCompteur_2 + newValueCompteur;
			}	

			sensTmp = 0;
			if (changementSens_2) {
				sensTmp = 1;
				if (BIT_TEST(signal ,0)) { /* Car le compteur compte sur un front montant */
					valueCompteur--; //valueCompteur>0 car on vient d'incrementer
					if (valueCompteur == 255) {
						sensTmp = 0;
						valueCompteur = 1;						
					}
				}
			}
				
			if (sens_2 != sensTmp) 
				compteur_2 += valueCompteur;				
			else
				compteur_2 -= valueCompteur;
			
			lastValueCompteur_2 = newValueCompteur;

			changementSens_2 = 0;

			/* Test de la RS232 */
			RS232_INPUT();

			/* Interruption set to 1*/
			interruption = 1;
		}
		while (!temps);

		/* on demance le calcul du PID actuel (contenu dans pid) */	
		calculPID();

		temps = 0;	

		/* On re-compte tout de suite apres */
		valueCompteur = 128;


	} while (1);

}

#SEPARATE signed long sommeLimite32000(signed long &valeur1, signed long &valeur2) {

	signed long temp;

	// Somme Limite a -+ 32000
	temp = valeur1 + valeur2;
	if ((valeur1>0) && (valeur2>0) && (temp<=0))  {
		return 32000;
	} else {
		if ((valeur1<0) && (valeur2<0) && (temp>=0)) 
			return -32000;
		else
			return temp;
	}
}

#SEPARATE signed long multiLong(unsigned long &valeur1, short signe_valeur1,  unsigned int &valeur2_fort) {
 
	signed long temp;

	temp = (valeur1>>8) * valeur2_fort;
	if (temp>255) {
		if (signe_valeur1)
			temp = 32760;
		else
			temp = -32760;
	} else {
		temp = (temp<<8);
		//temp += (valeur1>>8) * valeur2_faible ;
		temp += ((long)(valeur1&255)) * valeur2_fort ;
		//temp += (int)(((valeur1&255) * valeur2_faible)>>8);
		if (!signe_valeur1)
			temp = -temp;
	}
	return temp;
}


// Calcul du PID
#SEPARATE void calculPID() 
{
	signed long e_signe;
	signed long temp;
	signed long temp2;
	signed long PWM_ADD;
	signed long Si;
	signed long max;

	unsigned long  e;
	short signe_e;
	unsigned long  non_signe_Si;
	short signe_Si;

	unsigned int Coef;

	// (PWM) = Kp * e + Te /Ti * (Si += e ) + Td/Te (e(t)-e(t-Te)

	if (pid) {
		if (freinMoteur1) {
			Si1 = 0;
			OUTPUT_HIGH(PIN_B4);
			delay_cycles(1);
			OUTPUT_HIGH(PIN_B3);
			SET_PWM1_DUTY_10b(1000);
			compteur_1_global += compteur_1;
			compteur_1 = 0;
			return;
		}
		if (roueLibreMoteur1) {
			Si1 = 0;
			SET_PWM1_DUTY_10b(0);
			compteur_1_global += compteur_1;
			compteur_1 = 0;
			return;
		}
	}
	else {
		if (freinMoteur2) {
			Si2 = 0;
			OUTPUT_HIGH(PIN_B2);
			delay_cycles(1);
			OUTPUT_HIGH(PIN_B1);
			SET_PWM2_DUTY_10b(1000);
			compteur_2_global += compteur_2;
			compteur_2 = 0;
			return;
		}
		if (roueLibreMoteur2) {
			Si2 = 0;
			SET_PWM2_DUTY_10b(0);
			compteur_2_global += compteur_2;
			compteur_2 = 0;
			return;
		}
	}

	// Calcul de l'erreur
	if (pid) {
		Si = Si1;
		if (sensConsigne1)
			e_signe = compteur_1 - (signed long)consigne1;
		else
			e_signe = compteur_1 + (signed long)consigne1;
		lastError1 = e_signe;
		max = ((signed long)READ_EEPROM(LIMITE_SI_MOTEUR1_fort))<<8;
		max += READ_EEPROM(LIMITE_SI_MOTEUR1_faible);
	}
        else {
		Si = Si2;
		if (sensConsigne2)
			e_signe = compteur_2 - (signed long)consigne2;
		else
			e_signe = compteur_2 + (signed long)consigne2;
		lastError2 = e_signe;
		max = ((signed long)READ_EEPROM(LIMITE_SI_MOTEUR2_fort))<<8;
		max += READ_EEPROM(LIMITE_SI_MOTEUR2_faible);
	}

	// Emission de l'erreur si c'est demande
	if (EnvoisErreur!=0) {
		if (pid == EnvoisErreurPid) {
			bufferEmission2 = e_signe>>8;
			bufferEmission3 = e_signe & 0xFF;
			sizeBuffer = 2;
			EnvoisErreur--;
			if (EnvoisErreur==0) {
				if (EnvoisErreurPid)
					roueLibreMoteur1 = 1;
				else
					roueLibreMoteur2 = 1;			
			}
		}
	}
	

	if (e_signe<0) {
		e = -e_signe;
		signe_e = 0;
	}
	else  {
		e = e_signe;
		signe_e = 1;
	}
	

	// Si : Somme des erreurs. Limite a -+ 32000
	
	Si = sommeLimite32000(Si, e_signe);

	if (Si>max)
		Si = max;
	if (Si<-max)
		Si = -max;

	if (Si<0) {
		non_signe_Si = -Si;
		signe_Si = 0;
	}
	else  {
		non_signe_Si = Si;
		signe_Si = 1;
	}


	// Charge les coef du terme proportionnel depuis l'EEPROM
	if (pid) {
		Coef = READ_EEPROM(EEPROM_KP_MOTEUR1);
	} else {
		Coef = READ_EEPROM(EEPROM_KP_MOTEUR2);
	}

	// Calcul du terme proportionnel
	PWM_ADD = multiLong(e, signe_e, Coef);

	// Charge les coef du terme integral depuis l'EEPROM
	if (pid) {
		Coef = READ_EEPROM(EEPROM_INVKI_MOTEUR1);
	} else {
		Coef = READ_EEPROM(EEPROM_INVKI_MOTEUR2);
	}
	
	// Calcul du terme integral
	temp = multiLong(non_signe_Si, signe_Si, Coef);

	// Somme des termes. Limite a -+ 32000
	PWM_ADD = sommeLimite32000(PWM_ADD, temp);

	if (PWM_ADD>1000)
		PWM_ADD = 1000;
	if (PWM_ADD<-1000)
		PWM_ADD = -1000;

	if (pid) {
		Si1 = Si;
		if (PWM_ADD>=0) {
			OUTPUT_HIGH(PIN_B4);
			delay_cycles(1);
			OUTPUT_LOW(PIN_B3);
			SET_PWM1_DUTY_10b(PWM_ADD);
		}
		else {
			OUTPUT_LOW(PIN_B4);
			delay_cycles(1);
			OUTPUT_HIGH(PIN_B3);
			SET_PWM1_DUTY_10b(-PWM_ADD);
		}	
		compteur_1_global += compteur_1;
		compteur_1 = 0;
		lastPWM1 = PWM_ADD;
	}
	else {
		Si2 = Si;
		if (PWM_ADD>=0) {

			OUTPUT_HIGH(PIN_B2);
			delay_cycles(1);
			OUTPUT_LOW(PIN_B1);
			SET_PWM2_DUTY_10b(PWM_ADD);
			}
		else {
			OUTPUT_LOW(PIN_B2);
			delay_cycles(1);
			OUTPUT_HIGH(PIN_B1);
			SET_PWM2_DUTY_10b(-PWM_ADD);
		}
		compteur_2_global += compteur_2;
		compteur_2 = 0;
		lastPWM2 = PWM_ADD;
	}	

}


/* PWM1 must be between 0 and 1023*/
void SET_PWM1_DUTY_10b (signed long PWM1) {

#BIT CCP1X = 0x17.5
#BIT CCP1Y = 0x17.4
#BYTE	CCPR1L = 0x15	

CCPR1L = (int)(PWM1>>2);
IF (PWM1 & 1)
	CCP1Y = 1;
else
	CCP1Y = 0;

IF (PWM1 & 2)
	CCP1X = 1;
else
	CCP1X = 0;

}

void SET_PWM2_DUTY_10b (signed long PWM2) {

#BIT CCP2X = 0x1D.5
#BIT CCP2Y = 0x1D.4
#BYTE	CCPR2L = 0x1B	

CCPR2L = (int)(PWM2>>2);
IF (PWM2 & 1)
	CCP2Y = 1;
else
	CCP2Y = 0;

IF (PWM2 & 2)
	CCP2X = 1;
else
	CCP2X = 0;
}


/*
Void InitLcd()
{
	char VarSend;
				// System Set
	VarSend= 0x30; 							// Mode 4 bit 
	Lcd_Send(VarSend);						// poids fort
	VarSend= 0x20;							// 2 Lignes
	Lcd_Send(VarSend);						// poids faible commande
				// Set ON/OFF
	VarSend= 0x00; 							// display On, curseur On, Blink On
	Lcd_Send(VarSend);
	VarSend = 0xE0;							// poids faible ecran On, Cursor On, Blink OFF
	Lcd_Send(VarSend);	
	delay_ms(2);
	Lcd_Clear();
				// Set entry mode
	VarSend = 0x00;
	Lcd_Send(VarSend);
	VArSend = 0x60;							// poids faible
	Lcd_Send(VarSend);
	Lcd_Clear();
	delay_ms(2);


}
void Lcd_Line1()
{
									// position du curseur sur la ligne 1
	char VarToSend;
	VarToSend=0x80;							// poids fort de 0x80
	Lcd_Send(VarToSend);
	VarToSend=0x00;							// poids faible
	Lcd_Send(VarToSend);
}
void Lcd_Line2()
{									// position du curseur sur la ligne 2
	char VarToSend;
	VarToSend=0xC0;							// poids fort de 0xC0 ,position 64
	Lcd_Send(VarToSend);
	VarToSend=0x00;							// poids faible
	Lcd_Send(VarToSend);
}
*/
/*
void Lcd_Line3()
{									// position du curseur sur la ligne 3
	char VarToSend;
	VarToSend=0x90;							// poids fort de 0x94 , position 20
	Lcd_Send(VarToSend);
	VarToSend=0x40;							// poids faible
	Lcd_Send(VarToSend);
}
*//*
void Lcd_Line4()
{									// position du curseur sur la ligne 4
	char VarToSend;
	VarToSend=0xD0;							// poids fort de 0xD4, position 84
	Lcd_Send(VarToSend);
	VarToSend=0x40;							// poids faible
	Lcd_Send(VarToSend);
}*/
/*
void Lcd_Send_noWait(char VarSend)
{									// Envoi de caractere sans tempo
	LCD_DATA_PORT= VarSend;  					// masquage des poids faible
	delay_cycles(1); 						// equivalent  un nop
	bit_set(LCD_DATA_PORT,2); 					// mise  1 de E (validation)
	delay_cycles(1);
	delay_cycles(1);
	delay_cycles(1); 						// delai de 450 ns a respecter ... 
	bit_clear(LCD_DATA_PORT,2);					// mise  0 de E
}

void Lcd_Send(char VarSend)
{									// input VarSend => contient l octect de command  envoyer.
	Lcd_send_noWait(VarSend);
									// envoi d un octets sur l ecran de l afficheur LCD
	delay_us(40); 							// attente de 40us mini pour une commande cf doc
									// envoi des poids faible  l ecran
}

Void Lcd_Clear()
{
									// effacement de l ecran
	char VarToSend;							// cmd=0x01;
	VarToSend= 0x00;						// envoi poids fort
	Lcd_Send(VarToSend);						// appel de la fonction d envoi de commande
	VarToSend= 0x10;						// envoi poids faible
	Lcd_Send(VarToSend);
	delay_ms(2);							// attente necessaire pour la commande
}

void Lcd_Char(char CharSend)
{									// envoi d un charactere  l cran
	char VarToSend;							// variable interne  la fonction
	VarToSend= CharSend & 0xF0;					// masquage des poids faible
	VarToSend= VarToSend +0x01; 					// RS =1
	LCD_DATA_PORT= VarToSend;  					// masquage des poids faible
	delay_cycles(1); 						// equivalent  un nop
	bit_set(LCD_DATA_PORT,2); 					// mise  1 de E (validation)
	delay_cycles(1);
	delay_cycles(1);
	delay_cycles(1); 						// delai de 450 ns a respecter
	bit_clear(LCD_DATA_PORT,2);					// mise  0 de E

	VarToSend = CharSend <<4;					// decalage poids faible
	VarToSend = VarToSend & 0xF0;					// masquage poids faible
	VarToSend = VarToSend +1;					// position de RS=1
	LCD_DATA_PORT= VarToSend;	  				// masquage des poids faible
	delay_cycles(1); 						// equivalent  un nop
	bit_set(LCD_DATA_PORT,2); 					// mise  1 de E (validation)
	delay_cycles(1);
	delay_cycles(1);
	delay_cycles(1); 						// delai de 450 ns a respecter
	bit_clear(LCD_DATA_PORT,2);					// mise  0 de E
	delay_us(50);							// attente necessaire pour l ecran?
}
*/
/*
Void ReturnHome()
{
	Char VarSend;
	VarSend= 0x00;
	Lcd_Send(VarSend);
	VarSend= 0x20;
	Lcd_Send(VarSend);
}*//*
void Lcd_Printf(char *Chaine)
{
	// affichage d une chaine  l ecran du LCD.
	while(*Chaine != '\0')						// Affichage tant qu on est pas  la fin de la chaine
		{ 
		Lcd_Char(*Chaine++);
		}
}*//*
void Lcd_Cur_S_Right()
{
	char VarSend;								// Valeur pour shift droit 0x14
	VarSend= 0x10;	
	lcd_send(VarSend);
	VarSend= 0x40;
	Lcd_Send(VarSend);
}*/
/*
void Lcd_Cur_S_Left()
{
	char VarSend;								// Valeur pour shift gauche 0x10
	VarSend= 0x10;	
	lcd_send(VarSend);
	VarSend= 0x00;
	Lcd_Send(VarSend);
}*//*
void Lcd_Hexa(char VarSend)
{
										// affiche  l ecran la valeur Hexa du nombre
	char VarToSend;								// variable temporaire
	//Lcd_Char(0x30);								// affichage de 0x...
	Lcd_Char(0x78);
	if ((VarSend>>4) > 9) 
		VartoSend= 0x37+(VarSend>>4);			// envoi quartet poids fort
	else
		VartoSend= 0x30+(VarSend>>4);
	Lcd_Char(VarToSend);
	if ((VarSend & 0x0F) > 9)
		VartoSend= 0x37 +(VarSend & 0x0F);
	else
		VarToSend= 0x30+( VarSend  & 0x0F);					// envoi quartet poids faible
	Lcd_Char(VarToSend);
}*/
/*
void AffichageAlphabet()
{
	// Cette fonction va afficher l ensemble des caractere support par votre ecran
	char i;
	Lcd_Clear();
	Lcd_Line1();								// position en ligne 1
	for ( i=0x20;i<0xFF;i++)
		{
		if( (i%20) == 0)					// modulo 20 on efface ( cas d un ecran 4*20)
			Lcd_Clear();
		lcd_Char(i);						// clignotement de la led.
		delay_ms(400);
		}
}
*/
void delay_sec(int n)
{
	for(;n!=0;n--)
		delay_ms(1000);
}
/*
void Lcd_Binaire(char VarSend)
{
									// fonction d affichage d une valeur binaire
	char i=7;
	do
	{	if (bit_test(VarSend,i))
		Lcd_Char(0x31);
		else
		Lcd_Char(0x30);
	} while ( i-- != 0);
	Lcd_Char(0x62);	
}*/
/*
void DisplayHexaBinDeci(char Val)
{
	Lcd_Clear();							// effacement
	strcpy (string, "Valeur en");					// Affichage du titre
	Lcd_printf(string);
	Lcd_Line2();							// Position en ligne 2
	strcpy (string,"Hexa:");
	Lcd_Printf(string);
	Lcd_Hexa(Val);							// Affichage de la valeur en hexa
	Lcd_Line3();							// Position en ligne 3
	strcpy (string,"Binaire:");
	Lcd_Printf(string);
	Lcd_Binaire(Val);						//Affichage de la valeur en binaire
	Lcd_Line4();							// Position en ligne 4
	strcpy (string, "Deci:");					
	Lcd_Printf(string);
									// Pas encore fait
	Lcd_Decimal(Val);						// Affichage de la representation decimal

}*/
/*
void Lcd_Decimal(char ValSend)
{
	char val=0;							// Affichage de la valeur en Decimal
	while ( ValSend >=100)
	{
		valsend = valsend -100;
		val++;
	}
	Lcd_Char('0'+val);
	val=0;
	while(ValSend >=10)
	{
		Valsend= valsend -10;
		val++;	
	}
	Lcd_Char('0'+val);
	Lcd_Char('0'+ValSend);
	
}*/





